package com.example.smartgarden.rest.adapter;

import android.content.Context;

import com.example.smartgarden.BuildConfig;
import com.example.smartgarden.rest.interceptors.AuthInterceptor;

import okhttp3.OkHttpClient;
import okhttp3.logging.HttpLoggingInterceptor;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;

class BaseAdapter {

    private static final HttpLoggingInterceptor.Level LEVEL_LOG = HttpLoggingInterceptor.Level.BODY;
    private static Retrofit sRetrofit;
    private Context sContext;

    /**
     * Constructor
     *
     * @param baseUrl base API URL
     * @param context application context
     */

    BaseAdapter(String baseUrl, Context context) {
        sContext = context;
        init(baseUrl);
    }

    /**
     * Initialize retrofit instance if not exists. Otherwise, if baseUrl change, create new instance
     *
     * @param baseUrl base API URL
     */
    private void init(String baseUrl) {
        if (sRetrofit == null) {
            sRetrofit = getRetrofitInstance(baseUrl);
        } else {
            if (!sRetrofit.baseUrl().toString().equals(baseUrl)) {
                sRetrofit = getRetrofitInstance(baseUrl);
            }
        }
    }

    /**
     * Get retrofit builder instance
     *
     * @param baseUrl base API URL
     * @return retrofit builder instance
     */
    private Retrofit getRetrofitInstance(String baseUrl) {
        return new Retrofit.Builder().baseUrl(baseUrl)
                .client(getClient())
                .addConverterFactory(GsonConverterFactory.create())
                .build();
    }

    /**
     * Get OkHttpClient, install authorization and debug interceptor
     *
     * @return client builder instance
     */
    private OkHttpClient getClient() {
        OkHttpClient.Builder builderClientHttp = new OkHttpClient().newBuilder();

        builderClientHttp.addInterceptor(new AuthInterceptor(sContext));

        if (BuildConfig.DEBUG) {
            HttpLoggingInterceptor interceptor = new HttpLoggingInterceptor();
            interceptor.setLevel(LEVEL_LOG);
            builderClientHttp.addInterceptor(interceptor);
        }
        return builderClientHttp.build();
    }

    /**
     * Get retrofit service
     *
     * @param _class class paramter
     * @param <T>    generic type
     * @return retrofit service
     */
    <T> T createService(Class<T> _class) {
        return sRetrofit.create(_class);
    }
}
