package com.example.smartgarden.rest.adapter;

import android.content.Context;

import com.example.smartgarden.rest.model.Board;
import com.example.smartgarden.rest.model.Device;
import com.example.smartgarden.rest.model.Driver;
import com.example.smartgarden.rest.model.Log;
import com.example.smartgarden.rest.model.PostResponse;
import com.example.smartgarden.rest.model.Schedule;
import com.example.smartgarden.rest.model.User;
import com.example.smartgarden.rest.service.SmartGardenApiService;
import com.example.smartgarden.rest.settings.NetworkSettingsManager;

import java.util.List;

import retrofit2.Call;

public class SmartGardenAdapter extends BaseAdapter implements SmartGardenApiService {

    private SmartGardenApiService mSmartGardenApiService;

    /**
     * Constructor
     *
     * @param context application context
     */
    public SmartGardenAdapter(Context context) {
        super(new NetworkSettingsManager(context).getNetworkUrl(), context);
        mSmartGardenApiService = createService(SmartGardenApiService.class);
    }

    /**
     * Post register user
     *
     * @param user object to send to API
     * @return api call
     */
    @Override
    public Call<PostResponse> register(User user) {
        return mSmartGardenApiService.register(user);
    }

    /**
     * Post login unser
     *
     * @param user object to send to API
     * @return api call
     */
    @Override
    public Call<PostResponse> login(User user) {
        return mSmartGardenApiService.login(user);
    }

    /**
     * Post board user
     *
     * @param board object to send to API
     * @return api call
     */
    @Override
    public Call<PostResponse> addBoard(Board board) {
        return mSmartGardenApiService.addBoard(board);
    }

    /**
     * Post board device
     *
     * @param device object to send to API
     * @return api call
     */
    @Override
    public Call<PostResponse> addDevice(Device device) {
        return mSmartGardenApiService.addDevice(device);
    }

    /**
     * Post board schedule
     *
     * @param schedule object to send to API
     * @return api call
     */
    @Override
    public Call<PostResponse> addSchedule(Schedule schedule) {
        return mSmartGardenApiService.addSchedule(schedule);
    }


    /**
     * Get drivers
     *
     * @return api call
     */
    @Override
    public Call<List<Driver>> getDrivers() {
        return mSmartGardenApiService.getDrivers();
    }

    /**
     * Get driver
     *
     * @return api call
     */
    @Override
    public Call<Driver> getDriver(String id) {
        return mSmartGardenApiService.getDriver(id);
    }

    /**
     * Get boards
     *
     * @return api call
     */
    @Override
    public Call<List<Board>> getBoards() {
        return mSmartGardenApiService.getBoards();
    }

    /**
     * Get schedules
     *
     * @return api call
     */
    @Override
    public Call<List<Schedule>> getSchedules() {
        return mSmartGardenApiService.getSchedules();
    }

    /**
     * Get logs
     *
     * @return api call
     */
    @Override
    public Call<List<Log>> getLogs() {
        return mSmartGardenApiService.getLogs();
    }

    /**
     * Delete board
     *
     * @return api call
     */
    @Override
    public Call<PostResponse> deleteBoard(String id) {
        return mSmartGardenApiService.deleteBoard(id);
    }

    /**
     * Delete device
     *
     * @return api call
     */
    @Override
    public Call<PostResponse> deleteDevice(String boardId, String deviceId) {
        return mSmartGardenApiService.deleteDevice(boardId, deviceId);
    }

    /**
     * Delete schedule
     *
     * @return api call
     */
    @Override
    public Call<PostResponse> deleteSchedule(String id) {
        return mSmartGardenApiService.deleteSchedule(id);
    }
}
