package com.example.smartgarden.rest.interceptors;

import android.content.Context;

import com.example.smartgarden.rest.session.SessionManager;

import java.io.IOException;

import okhttp3.Interceptor;
import okhttp3.Request;
import okhttp3.Response;

public class AuthInterceptor implements Interceptor {

    private SessionManager mSessionManager;

    /**
     * Constructor
     *
     * @param context application context
     */
    public AuthInterceptor(Context context) {
        mSessionManager = new SessionManager(context);
    }

    /**
     * Interceptor for set authorization bearer for using JWT
     *
     * @param chain data to evaluate
     * @return request with header
     * @throws IOException
     */
    @Override
    public Response intercept(Chain chain) throws IOException {
        Request.Builder requestBuilder = chain.request().newBuilder();

        String token = mSessionManager.getAuthToken();

        if (token != null) {
            requestBuilder.addHeader("Authorization", "Bearer " + token);
        }

        return chain.proceed(requestBuilder.build());
    }
}
