package com.example.smartgarden.rest.service;

import com.example.smartgarden.rest.constants.ApiConstants;
import com.example.smartgarden.rest.model.Board;
import com.example.smartgarden.rest.model.Device;
import com.example.smartgarden.rest.model.Driver;
import com.example.smartgarden.rest.model.Log;
import com.example.smartgarden.rest.model.PostResponse;
import com.example.smartgarden.rest.model.Schedule;
import com.example.smartgarden.rest.model.User;

import java.util.List;

import retrofit2.Call;
import retrofit2.http.Body;
import retrofit2.http.DELETE;
import retrofit2.http.GET;
import retrofit2.http.POST;
import retrofit2.http.Query;

public interface SmartGardenApiService {

    /**
     * Post register api
     *
     * @param user user to register
     * @return post response
     */
    @POST(ApiConstants.SMARTGARDEN_USER_REGISTER)
    Call<PostResponse> register(@Body User user);

    /**
     * Post login api
     *
     * @param user user to login
     * @return post response
     */
    @POST(ApiConstants.SMARTGARDEN_USER_LOGIN)
    Call<PostResponse> login(@Body User user);

    /**
     * Post board api
     *
     * @param board board to add
     * @return post response
     */
    @POST(ApiConstants.SMARTGARDEN_ADD_BOARD)
    Call<PostResponse> addBoard(@Body Board board);

    /**
     * Post device api
     *
     * @param device device to add
     * @return post response
     */
    @POST(ApiConstants.SMARTGARDEN_ADD_DEVICE)
    Call<PostResponse> addDevice(@Body Device device);

    /**
     * Post schedule api
     *
     * @param schedule schedule to add
     * @return post response
     */
    @POST(ApiConstants.SMARTGARDEN_ADD_SCHEDULE)
    Call<PostResponse> addSchedule(@Body Schedule schedule);

    /**
     * Get drivers api
     *
     * @return get response
     */
    @GET(ApiConstants.SMARTGARDEN_GET_DRIVERS)
    Call<List<Driver>> getDrivers();

    /**
     * Get driver api
     *
     * @param id driver id
     * @return get response
     */
    @GET(ApiConstants.SMARTGARDEN_GET_DRIVER)
    Call<Driver> getDriver(@Query("id") String id);

    /**
     * Get boards api
     *
     * @return get response
     */
    @GET(ApiConstants.SMARTGARDEN_GET_BOARDS)
    Call<List<Board>> getBoards();

    /**
     * Get schedules api
     *
     * @return get response
     */
    @GET(ApiConstants.SMARTGARDEN_GET_SCHEDULES)
    Call<List<Schedule>> getSchedules();

    /**
     * Get logs api
     *
     * @return get response
     */
    @GET(ApiConstants.SMARTGARDEN_GET_LOGS)
    Call<List<Log>> getLogs();

    /**
     * Delete board api
     *
     * @param id board id
     * @return api delete response
     */
    @DELETE(ApiConstants.SMARTGARDEN_DELETE_BOARD)
    Call<PostResponse> deleteBoard(@Query("id") String id);

    /**
     * Delete device api
     *
     * @param boardId  board id
     * @param deviceId device id
     * @return api delete response
     */
    @DELETE(ApiConstants.SMARTGARDEN_DELETE_DEVICE)
    Call<PostResponse> deleteDevice(@Query("boardId") String boardId, @Query("deviceId") String deviceId);

    /**
     * Delete schedule api
     *
     * @param id schedule id
     * @return api delete response
     */
    @DELETE(ApiConstants.SMARTGARDEN_DELETE_SCHEDULE)
    Call<PostResponse> deleteSchedule(@Query("id") String id);
}
