package com.example.smartgarden.rest.session;

import android.content.Context;
import android.content.SharedPreferences;

import com.example.smartgarden.R;

public class SessionManager {

    private static final String USER_ID = "user_id";
    private static final String USER_TOKEN = "user_token";

    private SharedPreferences mPreferences;

    /**
     * Constructor
     *
     * @param context application context
     */
    public SessionManager(Context context) {
        mPreferences = context.getSharedPreferences(context.getString(R.string.app_name),
                Context.MODE_PRIVATE);
    }

    /**
     * Get stored user id
     *
     * @return stored user id
     */
    public String getUserId() {
        return mPreferences.getString(USER_ID, null);
    }

    /**
     * Get stored auth token
     *
     * @return stored auth token
     */
    public String getAuthToken() {
        return mPreferences.getString(USER_TOKEN, null);
    }

    /**
     * Set session data
     *
     * @param userId user id to store
     * @param token  user token to store
     */
    public void setSessionData(String userId, String token) {
        SharedPreferences.Editor editor = mPreferences.edit();
        editor.putString(USER_ID, userId);
        editor.putString(USER_TOKEN, token);
        editor.apply();
    }

    /**
     * Clean stored session data
     */
    public void cleanSessionData() {
        SharedPreferences.Editor editor = mPreferences.edit();
        editor.remove(USER_ID);
        editor.remove(USER_TOKEN);
        editor.apply();
    }
}
