package com.example.smartgarden.rest.settings;

import android.content.Context;
import android.content.SharedPreferences;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.constants.ApiConstants;

public class NetworkSettingsManager {

    private static final String NETWORK_IP = "network_ip";
    private static final String NETWORK_PORT = "network_port";

    private SharedPreferences mPreferences;

    /**
     * Constructor
     *
     * @param context application context
     */
    public NetworkSettingsManager(Context context) {
        mPreferences = context.getSharedPreferences(context.getString(R.string.app_name),
                Context.MODE_PRIVATE);

    }

    /**
     * Get stored network ip
     *
     * @return stored network ip
     */
    public String getNetworkIp() {
        return mPreferences.getString(NETWORK_IP, null);
    }

    /**
     * Get stored network port
     *
     * @return stored network port
     */
    public String getNetworkPort() {
        return mPreferences.getString(NETWORK_PORT, null);
    }

    /**
     * Get network url
     *
     * @return network api url
     */
    public String getNetworkUrl() {
        String ip = getNetworkIp();
        String port = getNetworkPort();

        if (ip == null || port == null) {
            return ApiConstants.BASE_SMARTGARDEN_URL;
        }

        StringBuilder urlBuilder = new StringBuilder();
        urlBuilder.append("http://");
        urlBuilder.append(getNetworkIp());
        urlBuilder.append(":");
        urlBuilder.append(getNetworkPort());

        return urlBuilder.toString();
    }

    /**
     * Set network settings
     *
     * @param ip   ip to store
     * @param port port to store
     */
    public void setNetworkSettings(String ip, String port) {
        SharedPreferences.Editor editor = mPreferences.edit();
        editor.putString(NETWORK_IP, ip);
        editor.putString(NETWORK_PORT, port);
        editor.apply();
    }
}
