package com.example.smartgarden.ui.activities;

import android.content.Intent;
import android.os.Bundle;
import android.view.MenuItem;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.Fragment;

import com.example.smartgarden.R;
import com.example.smartgarden.ui.constants.UiConstants;
import com.example.smartgarden.ui.fragments.AddBoardFragment;
import com.example.smartgarden.ui.fragments.AddDeviceFragment;
import com.example.smartgarden.ui.fragments.AddScheduleFragment;
import com.example.smartgarden.ui.fragments.DashboardFragment;
import com.example.smartgarden.ui.fragments.ExportImportFragment;
import com.example.smartgarden.ui.fragments.HowtoFragment;
import com.example.smartgarden.ui.fragments.LogsFragment;
import com.example.smartgarden.ui.model.MenuContent;

public class MenuDetailActivity extends AppCompatActivity {

    /**
     * onCreate method
     *
     * @param savedInstanceState saved instance state
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_menu_detail);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        // Show the Up button in the action bar.
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setDisplayHomeAsUpEnabled(true);
        }

        int itemId = getIntent().getIntExtra(UiConstants.ARG_ITEM_ID, MenuContent.ITEM_DASHBOARD);

        String title = MenuContent.ITEMS.get(itemId).title;
        setTitle(title.substring(0, 1).toUpperCase() + title.substring(1).toLowerCase());

        if (savedInstanceState == null) {
            Fragment currentFragment = null;

            switch (itemId) {
                case MenuContent.ITEM_DASHBOARD:
                    currentFragment = new DashboardFragment();
                    break;
                case MenuContent.ITEM_ADD_BOARD:
                    currentFragment = new AddBoardFragment();
                    break;
                case MenuContent.ITEM_ADD_DEVICE:
                    currentFragment = new AddDeviceFragment();
                    break;
                case MenuContent.ITEM_ADD_SCHEDULE:
                    currentFragment = new AddScheduleFragment();
                    break;
                case MenuContent.ITEM_EXPORT_IMPORT:
                    currentFragment = new ExportImportFragment();
                    break;
                case MenuContent.ITEM_LOG:
                    currentFragment = new LogsFragment();
                    break;
                case MenuContent.ITEM_HOW_TO:
                    currentFragment = new HowtoFragment();
                    break;
                default:
                    currentFragment = new DashboardFragment();
                    break;
            }

            getSupportFragmentManager().beginTransaction()
                    .add(R.id.menu_detail_container, currentFragment)
                    .commit();
        }
    }

    /**
     * onOptionsItemSelected method
     *
     * @param item menu item
     * @return menu if activity contains menu item
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == android.R.id.home) {
            navigateUpTo(new Intent(this, MenuListActivity.class));
            return true;
        }
        return super.onOptionsItemSelected(item);
    }
}
