package com.example.smartgarden.ui.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.model.Board;
import com.example.smartgarden.rest.model.Device;
import com.example.smartgarden.rest.model.Schedule;
import com.example.smartgarden.ui.interfaces.DeleteClickListener;

import java.util.ArrayList;
import java.util.List;

public class BoardAdapter extends RecyclerView.Adapter<BoardAdapter.ViewHolder> {

    private DeleteClickListener mOnDeleteClickedListener;

    private List<Board> mBoardsList;
    private List<Schedule> mScheduleList;

    /**
     * Constructor
     *
     * @param boardsList    list of boards
     * @param schedulesList list of schedules
     * @param listener      delete click listener
     */
    public BoardAdapter(List<Board> boardsList, List<Schedule> schedulesList,
                        DeleteClickListener listener) {

        this.mOnDeleteClickedListener = listener;
        this.mBoardsList = boardsList;
        this.mScheduleList = schedulesList;
    }

    /**
     * onCreateViewHolder method
     *
     * @param parent   parent view group
     * @param viewType view type
     * @return new view holder instance
     */
    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.boards_list_content, parent, false);
        return new ViewHolder(view);
    }

    /**
     * onBindViewHolder method
     *
     * @param holder   view holder
     * @param position bind position
     */
    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        holder.mBoardLabel.setText(mBoardsList.get(position).getName());

        List<Device> deviceList = mBoardsList.get(position).getDevices();
        List<Schedule> scheduleList = new ArrayList<>();

        for (Schedule schedule : mScheduleList) {
            if (schedule.getBoardId().equals(mBoardsList.get(position).getId())) {
                scheduleList.add(schedule);
            }
        }

        DeviceAdapter devicesAdapter = new DeviceAdapter(deviceList, mOnDeleteClickedListener);
        ScheduleAdapter routinesAdapter = new ScheduleAdapter(scheduleList, mOnDeleteClickedListener);

        holder.mDevicesView.setAdapter(devicesAdapter);
        holder.mSchedulesView.setAdapter(routinesAdapter);
    }

    /**
     * Count elements in board list
     *
     * @return boards list size
     */
    @Override
    public int getItemCount() {
        return mBoardsList.size();
    }

    class ViewHolder extends RecyclerView.ViewHolder {

        private TextView mBoardLabel;
        private ImageButton mDeleteButton;
        private RecyclerView mDevicesView;
        private RecyclerView mSchedulesView;

        /**
         * Constructor
         *
         * @param itemView item view
         */
        ViewHolder(@NonNull View itemView) {
            super(itemView);

            mBoardLabel = (TextView) itemView.findViewById(R.id.boardName);
            mDeleteButton = (ImageButton) itemView.findViewById(R.id.btnBoardDelete);
            mDevicesView = (RecyclerView) itemView.findViewById(R.id.recyclerViewDevice);
            mSchedulesView = (RecyclerView) itemView.findViewById(R.id.recyclerViewSchedule);

            mDeleteButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    mOnDeleteClickedListener.deleteBoardOnClick(mBoardsList.get(getAdapterPosition()).getId());
                }
            });
        }
    }
}
