package com.example.smartgarden.ui.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.model.Device;
import com.example.smartgarden.ui.constants.UiConstants;
import com.example.smartgarden.ui.interfaces.DeleteClickListener;

import java.util.List;

class DeviceAdapter extends RecyclerView.Adapter<DeviceAdapter.ViewHolder> {

    private DeleteClickListener mOnDeleteDeviceClicked;
    private List<Device> mDeviceList;

    /**
     * Constructor
     *
     * @param deviceList list of devices
     * @param listener   delete click listener
     */
    DeviceAdapter(List<Device> deviceList, DeleteClickListener listener) {
        this.mOnDeleteDeviceClicked = listener;
        this.mDeviceList = deviceList;
    }

    /**
     * onCreateViewHolder method
     *
     * @param parent   parent view group
     * @param viewType view type
     * @return new view holder instance
     */
    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.devices_list_content, parent, false);
        return new ViewHolder(view);
    }

    /**
     * onBindViewHolder method
     *
     * @param holder   view holder
     * @param position bind position
     */
    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        holder.mDeviceName.setText(mDeviceList.get(position).getName());

        if (mDeviceList.get(position).getType().equals(UiConstants.SENSOR_TAG)) {
            holder.mDeviceState.setText(mDeviceList.get(position).getLatestValue().toString());
        } else {
            holder.mDeviceState.setText(mDeviceList.get(position).getLatestValue() == 1 ?
                    holder.itemView.getContext().getString(R.string.message_actuator_on) :
                    holder.itemView.getContext().getString(R.string.message_actuator_off));
        }
    }

    /**
     * Count elements in devices list
     *
     * @return devices list size
     */
    @Override
    public int getItemCount() {
        return mDeviceList.size();
    }

    class ViewHolder extends RecyclerView.ViewHolder {

        private TextView mDeviceName;
        private TextView mDeviceState;

        /**
         * Constructor
         *
         * @param itemView item view
         */
        ViewHolder(@NonNull View itemView) {
            super(itemView);

            mDeviceName = (TextView) itemView.findViewById(R.id.labelName);
            mDeviceState = (TextView) itemView.findViewById(R.id.labelState);

            ImageButton mDeleteButton = (ImageButton) itemView.findViewById(R.id.btnDelete);
            mDeleteButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    mOnDeleteDeviceClicked.deleteDeviceOnClick(
                            mDeviceList.get(getAdapterPosition()).getBoardId(),
                            mDeviceList.get(getAdapterPosition()).getId());
                }
            });
        }
    }
}
