package com.example.smartgarden.ui.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.model.Log;

import java.text.SimpleDateFormat;
import java.util.List;

public class LogAdapter extends RecyclerView.Adapter<LogAdapter.ViewHolder> {

    private List<Log> mLogsList;

    /**
     * Constructor
     *
     * @param logsList list of logs
     */
    public LogAdapter(List<Log> logsList) {
        this.mLogsList = logsList;
    }

    /**
     * onCreateViewHolder method
     *
     * @param parent   parent view group
     * @param viewType view type
     * @return new view holder instance
     */
    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.logs_list_content, parent, false);
        return new ViewHolder(view);
    }

    /**
     * onBindViewHolder method
     *
     * @param holder   view holder
     * @param position bind position
     */
    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        SimpleDateFormat format = new SimpleDateFormat("HH:mm");

        holder.mFirstLine.setText(mLogsList.get(position).getEvent());
        holder.mSecondLine.setText(mLogsList.get(position).getDescription());
        holder.mDate.setText(format.format(mLogsList.get(position).getDate()));
    }

    /**
     * Count elements in logs list
     *
     * @return logs list size
     */
    @Override
    public int getItemCount() {
        return mLogsList.size();
    }

    static class ViewHolder extends RecyclerView.ViewHolder {

        private TextView mFirstLine;
        private TextView mSecondLine;
        private TextView mDate;

        /**
         * Constructor
         *
         * @param itemView item view
         */
        ViewHolder(@NonNull View itemView) {
            super(itemView);
            mFirstLine = (TextView) itemView.findViewById(R.id.firstLine);
            mSecondLine = (TextView) itemView.findViewById(R.id.secondLine);
            mDate = (TextView) itemView.findViewById(R.id.date);
        }
    }
}
