package com.example.smartgarden.ui.adapters;

import android.content.Context;
import android.content.Intent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;

import com.example.smartgarden.R;
import com.example.smartgarden.ui.activities.MenuDetailActivity;
import com.example.smartgarden.ui.activities.MenuListActivity;
import com.example.smartgarden.ui.constants.UiConstants;
import com.example.smartgarden.ui.fragments.AddBoardFragment;
import com.example.smartgarden.ui.fragments.AddDeviceFragment;
import com.example.smartgarden.ui.fragments.AddScheduleFragment;
import com.example.smartgarden.ui.fragments.DashboardFragment;
import com.example.smartgarden.ui.fragments.ExportImportFragment;
import com.example.smartgarden.ui.fragments.HowtoFragment;
import com.example.smartgarden.ui.fragments.LogsFragment;
import com.example.smartgarden.ui.model.MenuContent;

import java.util.List;

public class MenuAdapter extends RecyclerView.Adapter<MenuAdapter.ViewHolder> {

    private final MenuListActivity mParentActivity;
    private final List<MenuContent.MenuItem> mMenuList;
    private final boolean mTwoPane;
    private final View.OnClickListener mOnClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View view) {
            MenuContent.MenuItem item = (MenuContent.MenuItem) view.getTag();
            if (mTwoPane) {
                Fragment currentFragment = null;

                switch (item.id) {
                    case MenuContent.ITEM_DASHBOARD:
                        currentFragment = new DashboardFragment();
                        break;
                    case MenuContent.ITEM_ADD_BOARD:
                        currentFragment = new AddBoardFragment();
                        break;
                    case MenuContent.ITEM_ADD_DEVICE:
                        currentFragment = new AddDeviceFragment();
                        break;
                    case MenuContent.ITEM_ADD_SCHEDULE:
                        currentFragment = new AddScheduleFragment();
                        break;
                    case MenuContent.ITEM_EXPORT_IMPORT:
                        currentFragment = new ExportImportFragment();
                        break;
                    case MenuContent.ITEM_LOG:
                        currentFragment = new LogsFragment();
                        break;
                    case MenuContent.ITEM_HOW_TO:
                        currentFragment = new HowtoFragment();
                        break;
                    default:
                        currentFragment = new DashboardFragment();
                        break;
                }

                mParentActivity.getSupportFragmentManager().beginTransaction()
                        .replace(R.id.menu_detail_container, currentFragment)
                        .commit();
            } else {
                Context context = view.getContext();
                Intent intent = new Intent(context, MenuDetailActivity.class);
                intent.putExtra(UiConstants.ARG_ITEM_ID, item.id);
                context.startActivity(intent);
            }
        }
    };

    /**
     * Constructor
     *
     * @param parent  parent activity
     * @param items   list of menu items
     * @param twoPane two pane view mode
     */
    public MenuAdapter(MenuListActivity parent, List<MenuContent.MenuItem> items, boolean twoPane) {
        mMenuList = items;
        mParentActivity = parent;
        mTwoPane = twoPane;
    }

    /**
     * onCreateViewHolder method
     *
     * @param parent   parent view group
     * @param viewType view type
     * @return new view holder instance
     */
    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.menu_list_content, parent, false);
        return new ViewHolder(view);
    }

    /**
     * onBindViewHolder method
     *
     * @param holder   view holder
     * @param position bind position
     */
    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        holder.mTitle.setText(mMenuList.get(position).title);

        holder.itemView.setTag(mMenuList.get(position));
        holder.itemView.setOnClickListener(mOnClickListener);
    }

    /**
     * Count elements in menu list
     *
     * @return menu list size
     */
    @Override
    public int getItemCount() {
        return mMenuList.size();
    }

    static class ViewHolder extends RecyclerView.ViewHolder {

        private TextView mTitle;

        /**
         * Constructor
         *
         * @param itemView item view
         */
        ViewHolder(@NonNull View itemView) {
            super(itemView);
            mTitle = (TextView) itemView.findViewById(R.id.title);
        }
    }
}
