package com.example.smartgarden.ui.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.model.Parameter;
import com.example.smartgarden.rest.model.Schedule;
import com.example.smartgarden.ui.constants.UiConstants;
import com.example.smartgarden.ui.interfaces.DeleteClickListener;

import java.util.List;

public class ScheduleAdapter extends RecyclerView.Adapter<ScheduleAdapter.ViewHolder> {

    private DeleteClickListener mOnDeleteScheduleClicked;
    private List<Schedule> mScheduleList;

    /**
     * Constructor
     *
     * @param scheduleList list of schedules
     * @param listener     delete click listener
     */
    ScheduleAdapter(List<Schedule> scheduleList, DeleteClickListener listener) {
        this.mOnDeleteScheduleClicked = listener;
        this.mScheduleList = scheduleList;
    }

    /**
     * onCreateViewHolder method
     *
     * @param parent   parent view group
     * @param viewType view type
     * @return new view holder instance
     */
    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.schedules_list_content, parent, false);
        return new ViewHolder(view);
    }

    /**
     * onBindViewHolder method
     *
     * @param holder   view holder
     * @param position bind position
     */
    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        holder.mScheduleName.setText(mScheduleList.get(position).getName());

        Parameter parameters = mScheduleList.get(position).getParameters();

        StringBuilder builder = new StringBuilder();

        if (mScheduleList.get(position).getType().equals(UiConstants.TRIGGER_DATE)) {
            builder.append(mScheduleList.get(position).getActuatorName());
            builder.append(" from ");
            builder.append(parameters.getStartTime());
            builder.append(" to ");
            builder.append(parameters.getEndTime());
        } else {
            builder.append(mScheduleList.get(position).getActuatorName());
            builder.append(" when ");
            builder.append(parameters.getSensorName());
            builder.append(" ");
            builder.append(parameters.getOperator());
            builder.append(" ");
            builder.append(parameters.getValue());
        }

        holder.mScheduleInfo.setText(builder.toString());
    }

    /**
     * Count elements in schedules list
     *
     * @return schedules list size
     */
    @Override
    public int getItemCount() {
        return mScheduleList.size();
    }

    class ViewHolder extends RecyclerView.ViewHolder {

        private TextView mScheduleName;
        private TextView mScheduleInfo;

        /**
         * Constructor
         *
         * @param itemView item view
         */
        ViewHolder(@NonNull View itemView) {
            super(itemView);

            mScheduleName = (TextView) itemView.findViewById(R.id.labelName);
            mScheduleInfo = (TextView) itemView.findViewById(R.id.labelInfo);

            ImageButton mDeleteButton = (ImageButton) itemView.findViewById(R.id.btnDelete);
            mDeleteButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    mOnDeleteScheduleClicked.deleteScheduleOnClick(mScheduleList.get(getAdapterPosition()).getId());
                }
            });
        }
    }
}
