package com.example.smartgarden.ui.dialogs;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.settings.NetworkSettingsManager;
import com.example.smartgarden.ui.interfaces.ServerDialogClickListener;

public class ServerDialog extends Dialog implements View.OnClickListener {

    private NetworkSettingsManager mNetworkSettingsManager;
    private EditText mIp;
    private EditText mPort;
    private ServerDialogClickListener mServerDialogListener;

    /**
     * Constructor
     *
     * @param context  application context
     * @param listener server dialog click listener
     */
    public ServerDialog(Context context, ServerDialogClickListener listener) {
        super(context);
        mServerDialogListener = listener;
    }

    /**
     * onCreate method
     *
     * @param savedInstanceState instance state
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.dialog_server);

        getWindow().setLayout(ViewGroup.LayoutParams.MATCH_PARENT,
                ViewGroup.LayoutParams.WRAP_CONTENT);

        mNetworkSettingsManager = new NetworkSettingsManager(getContext());

        mIp = (EditText) findViewById(R.id.server);
        mPort = (EditText) findViewById(R.id.port);

        Button mPositiveButton = (Button) findViewById(R.id.btnConfirm);
        Button mNegativeButton = (Button) findViewById(R.id.btnCancel);

        mPositiveButton.setOnClickListener(this);
        mNegativeButton.setOnClickListener(this);

        setExistingParameters();
    }

    /**
     * onClick method
     *
     * @param v view clicked
     */
    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnConfirm:
                String ip = mIp.getText().toString();
                String port = mPort.getText().toString();

                if (TextUtils.isEmpty(ip)) {
                    mIp.setError(getContext().getString(R.string.message_required_field));
                    return;
                } else {
                    mIp.setError(null);
                }

                if (TextUtils.isEmpty(port)) {
                    mPort.setError(getContext().getString(R.string.message_required_field));
                    return;
                } else {
                    mPort.setError(null);
                }

                mNetworkSettingsManager.setNetworkSettings(mIp.getText().toString(),
                        mPort.getText().toString());

                mServerDialogListener.saveButtonOnClick();
                break;
            case R.id.btnCancel:
                mServerDialogListener.cancelButtonOnClick();
                break;
            default:
                break;
        }
        dismiss();
    }

    /**
     * Set existing parameters in fields
     */
    private void setExistingParameters() {
        String ip = mNetworkSettingsManager.getNetworkIp();
        String port = mNetworkSettingsManager.getNetworkPort();

        if (ip != null) {
            mIp.setText(ip);
        }

        if (port != null) {
            mPort.setText(port);
        }
    }
}
