package com.example.smartgarden.ui.fragments;

import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.Spinner;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.adapter.SmartGardenAdapter;
import com.example.smartgarden.rest.constants.ApiConstants;
import com.example.smartgarden.rest.model.Board;
import com.example.smartgarden.rest.model.Device;
import com.example.smartgarden.rest.model.Driver;
import com.example.smartgarden.rest.model.Port;
import com.example.smartgarden.rest.model.PostResponse;
import com.example.smartgarden.ui.constants.UiConstants;
import com.example.smartgarden.ui.utils.Utils;
import com.google.android.material.textfield.TextInputLayout;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class AddDeviceFragment extends Fragment implements View.OnClickListener,
        AdapterView.OnItemSelectedListener {

    private Context mContext;

    private SmartGardenAdapter mApiService;
    private FrameLayout mLoadingFrame;
    private TextInputLayout mInputIdentification;
    private TextInputLayout mInputName;
    private Spinner mBoardSpinner;
    private Spinner mTypeSpinner;
    private Spinner mLocationSpinner;

    private ArrayAdapter<Board> mBoardAdapter;
    private ArrayAdapter<Port> mLocationAdapter;

    private List<Board> mBoardList;
    private List<String> mTypeList;
    private List<Port> mInputPortList;
    private List<Port> mOutputPortList;

    /**
     * Constructor
     */
    public AddDeviceFragment() {
        // Required empty public constructor
    }

    /**
     * Fragment new instance
     *
     * @return AddDeviceFragment instance
     */
    public static AddDeviceFragment newInstance() {
        return new AddDeviceFragment();
    }

    /**
     * onAttact method
     *
     * @param context application context
     */
    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        mContext = context;
    }

    /**
     * onCreate method
     *
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mApiService = new SmartGardenAdapter(mContext);
        mBoardList = new ArrayList<>();
        mTypeList = new ArrayList<>();
        mInputPortList = new ArrayList<>();
        mOutputPortList = new ArrayList<>();

        mTypeList.add(UiConstants.SENSOR_TAG);
        mTypeList.add(UiConstants.ACTUATOR_TAG);
    }

    /**
     * onCreateView method
     *
     * @param inflater           layout inflater
     * @param container          viewgroup container
     * @param savedInstanceState saved instance state
     * @return inflated view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_add_device, container, false);

        mLoadingFrame = view.findViewById(R.id.loadingFrame);
        mInputIdentification = view.findViewById(R.id.layoutIdentification);
        mInputName = view.findViewById(R.id.layoutName);
        mBoardSpinner = view.findViewById(R.id.spinnerBoard);
        mTypeSpinner = view.findViewById(R.id.spinnerType);
        mLocationSpinner = view.findViewById(R.id.spinnerLocation);

        Button saveButton = view.findViewById(R.id.btnSave);
        Button resetButton = view.findViewById(R.id.btnReset);

        Button autoIdentificator = view.findViewById(R.id.btnAutoId);
        ImageButton boardInfoButton = view.findViewById(R.id.btnInfoBoard);
        ImageButton typeInfoButton = view.findViewById(R.id.btnInfoType);
        ImageButton locationInfoButton = view.findViewById(R.id.btnInfoLocation);

        autoIdentificator.setOnClickListener(this);
        saveButton.setOnClickListener(this);
        resetButton.setOnClickListener(this);
        boardInfoButton.setOnClickListener(this);
        typeInfoButton.setOnClickListener(this);
        locationInfoButton.setOnClickListener(this);

        mBoardSpinner.setOnItemSelectedListener(this);
        mTypeSpinner.setOnItemSelectedListener(this);

        return view;
    }

    /**
     * onViewCreated method
     *
     * @param view               created view
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        mBoardAdapter = new ArrayAdapter<>(mContext,
                android.R.layout.simple_list_item_1, mBoardList);
        mBoardAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mBoardSpinner.setAdapter(mBoardAdapter);

        ArrayAdapter<String> mTypeAdapter = new ArrayAdapter<>(mContext,
                android.R.layout.simple_list_item_1, mTypeList);
        mTypeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mTypeSpinner.setAdapter(mTypeAdapter);

        mLocationAdapter = new ArrayAdapter<>(mContext,
                android.R.layout.simple_list_item_1, mInputPortList);
        mLocationAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mLocationSpinner.setAdapter(mLocationAdapter);

        showLoadingFrame();
        getApiBoards();
    }

    /**
     * onClick method
     *
     * @param v clicked view
     */
    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnAutoId:
                onAutoIdButtonClicked();
                break;
            case R.id.btnSave:
                onSaveButtonClicked();
                break;
            case R.id.btnReset:
                onResetButtonClicked();
                break;
            case R.id.btnInfoBoard:
                onBoardInfoButtonClicked();
                break;
            case R.id.btnInfoType:
                onTypeInfoButtonClicked();
                break;
            case R.id.btnInfoLocation:
                onLocationLInfoButtonClicked();
                break;
        }
    }

    /**
     * onItemSelected method
     *
     * @param parent   parent view
     * @param view     selected view
     * @param position item position
     * @param id       item id
     */
    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
        switch (parent.getId()) {
            case R.id.spinnerBoard:
                onBoardSpinnerSelectionChange(position);
                break;
            case R.id.spinnerType:
                onTypeSpinnerSelectionChange(position);
                break;
        }
    }

    @Override
    public void onNothingSelected(AdapterView<?> parent) {

    }

    /**
     * Get api boards request
     */
    private void getApiBoards() {
        Call<List<Board>> boardsCall = mApiService.getBoards();
        boardsCall.enqueue(new Callback<List<Board>>() {
            @Override
            public void onResponse(Call<List<Board>> call, Response<List<Board>> response) {
                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        if (response.body() != null) {
                            mBoardList = response.body();

                            mBoardAdapter.clear();
                            mBoardAdapter.addAll(mBoardList);
                            mBoardAdapter.notifyDataSetChanged();
                        }

                        if (mBoardList.size() > 0) {
                            getApiDriver(mBoardList.get(0).getDriverId());
                        } else {
                            hideLoadingFrame();
                            showToast(getString(R.string.message_data_loaded));
                        }
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        hideLoadingFrame();
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        hideLoadingFrame();
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<List<Board>> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_getting_boards));
            }
        });
    }

    /**
     * Get api driver request
     *
     * @param driverId driver id
     */
    private void getApiDriver(String driverId) {
        Call<Driver> driverCall = mApiService.getDriver(driverId);
        driverCall.enqueue(new Callback<Driver>() {
            @Override
            public void onResponse(Call<Driver> call, Response<Driver> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        if (response.body() != null) {
                            mInputPortList = response.body().getInputPorts();
                            mOutputPortList = response.body().getOutputPorts();
                        }

                        Board selectedBoard = (Board) mBoardSpinner.getSelectedItem();
                        for (Device device : selectedBoard.getDevices()) {
                            mInputPortList.remove(device.getPort());
                            mOutputPortList.remove(device.getPort());
                        }

                        mLocationAdapter.clear();
                        mLocationAdapter.addAll(mInputPortList);
                        mLocationAdapter.notifyDataSetChanged();

                        showToast(getString(R.string.message_data_loaded));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<Driver> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_getting_ports));
            }
        });
    }

    /**
     * Post api device request
     *
     * @param device device to post
     */
    private void postApiDevice(Device device) {
        Call<PostResponse> addDeviceCall = mApiService.addDevice(device);
        addDeviceCall.enqueue(new Callback<PostResponse>() {
            @Override
            public void onResponse(Call<PostResponse> call, Response<PostResponse> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_CREATED:
                        showToast(getString(R.string.message_device_added));
                        clearFields();
                        getApiBoards();
                        break;
                    case ApiConstants.API_CODE_CONFLICT:
                        showToast(getString(R.string.message_device_duplicated));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<PostResponse> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_adding_device));
            }
        });
    }

    /**
     * Callback for setting automatic id
     */
    private void onAutoIdButtonClicked() {
        mInputIdentification.getEditText().setText(Utils.getAutomaticId());
    }

    /**
     * Callback for save button clicked
     */
    private void onSaveButtonClicked() {
        String id = mInputIdentification.getEditText().getText().toString();
        String name = mInputName.getEditText().getText().toString();
        Board board = (Board) mBoardSpinner.getSelectedItem();
        String type = (String) mTypeSpinner.getSelectedItem();
        Port port = (Port) mLocationSpinner.getSelectedItem();

        if (TextUtils.isEmpty(id)) {
            mInputIdentification.setError(getString(R.string.message_required_field));
            return;
        } else {
            mInputIdentification.setError(null);
        }

        if (TextUtils.isEmpty(name)) {
            mInputName.setError(getString(R.string.message_required_field));
            return;
        } else {
            mInputName.setError(null);
        }

        if (board == null) {
            showToast(getString(R.string.message_board_must_be_selected));
            return;
        }

        if (type == null) {
            showToast(getString(R.string.message_type_must_be_selected));
            return;
        }

        if (port == null) {
            showToast(getString(R.string.message_port_must_be_selected));
            return;
        }

        Device device = new Device();
        device.setId(id);
        device.setName(name);
        device.setBoardId(board.getId());
        device.setPort(port);
        device.setType(type);
        device.setLatestValue(-1);

        showLoadingFrame();
        postApiDevice(device);
    }

    /**
     * Callback for reset button clicked
     */
    private void onResetButtonClicked() {
        clearFields();
    }

    /**
     * Callback for board info button clicked
     */
    private void onBoardInfoButtonClicked() {
        showToast(getString(R.string.message_info_board));
    }

    /**
     * Callback for type info button clicked
     */
    private void onTypeInfoButtonClicked() {
        showToast(getString(R.string.message_info_type));
    }

    /**
     * Callback for location info button clicked
     */
    private void onLocationLInfoButtonClicked() {
        showToast(getString(R.string.message_info_location));
    }

    /**
     * Callback for board spinner selection change
     *
     * @param position selected spinner board position
     */
    private void onBoardSpinnerSelectionChange(int position) {
        clearTypeSpinner();
        clearLocationSpinner();

        showLoadingFrame();
        getApiDriver(mBoardList.get(position).getDriverId());
    }

    /**
     * Callback for type spinner selection change
     *
     * @param position selected spinner type position
     */
    private void onTypeSpinnerSelectionChange(int position) {
        clearLocationSpinner();

        if (mTypeList.get(position).equals(UiConstants.SENSOR_TAG)) {
            mLocationAdapter.clear();
            mLocationAdapter.addAll(mInputPortList);
            mLocationAdapter.notifyDataSetChanged();
        } else {
            mLocationAdapter.clear();
            mLocationAdapter.addAll(mOutputPortList);
            mLocationAdapter.notifyDataSetChanged();
        }
    }

    /**
     * Show loading frame
     */
    private void showLoadingFrame() {
        mLoadingFrame.setVisibility(View.VISIBLE);
    }

    /**
     * Hide loading frame
     */
    private void hideLoadingFrame() {
        mLoadingFrame.setVisibility(View.GONE);
    }

    /**
     * Show toast with message
     *
     * @param message data to show
     */
    private void showToast(String message) {
        Toast.makeText(mContext, message, Toast.LENGTH_SHORT).show();
    }

    /**
     * Clear board spinner field
     */
    private void clearBoardSpinner() {
        if (mBoardSpinner.getCount() > 0) {
            mBoardSpinner.setSelection(0);
        }
    }

    /**
     * Clear type spinner field
     */
    private void clearTypeSpinner() {
        if (mTypeSpinner.getCount() > 0) {
            mTypeSpinner.setSelection(0);
        }
    }

    /**
     * Clear location spinner field
     */
    private void clearLocationSpinner() {
        if (mLocationSpinner.getCount() > 0) {
            mLocationSpinner.setSelection(0);
        }
    }

    /**
     * Clear all editable fields
     */
    private void clearFields() {
        mInputIdentification.getEditText().getText().clear();
        mInputName.getEditText().getText().clear();

        clearBoardSpinner();
        clearTypeSpinner();
        clearLocationSpinner();
    }
}
