package com.example.smartgarden.ui.fragments;

import android.app.TimePickerDialog;
import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TimePicker;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.example.smartgarden.R;
import com.example.smartgarden.rest.adapter.SmartGardenAdapter;
import com.example.smartgarden.rest.constants.ApiConstants;
import com.example.smartgarden.rest.model.Board;
import com.example.smartgarden.rest.model.Device;
import com.example.smartgarden.rest.model.Parameter;
import com.example.smartgarden.rest.model.PostResponse;
import com.example.smartgarden.rest.model.Schedule;
import com.example.smartgarden.rest.session.SessionManager;
import com.example.smartgarden.ui.constants.UiConstants;
import com.example.smartgarden.ui.utils.Utils;
import com.google.android.material.textfield.TextInputLayout;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class AddScheduleFragment extends Fragment implements View.OnClickListener,
        AdapterView.OnItemSelectedListener {

    private Context mContext;

    private SmartGardenAdapter mApiService;
    private SessionManager mSessionManager;

    private FrameLayout mLoadingFrame;
    private TextInputLayout mInputIdentification;
    private TextInputLayout mInputName;
    private Spinner mBoardSpinner;
    private Spinner mActuatorSpinner;
    private Spinner mTriggerTypeSpinner;
    private Spinner mSensorSpinner;
    private LinearLayout mSensorLayout;
    private RadioGroup mRadioGroupOperator;
    private TextInputLayout mTriggerValue;
    private TextInputLayout mStartDate;
    private TextInputLayout mEndDate;
    private TimePickerDialog mTimePicker;

    private ArrayAdapter<Board> mBoardAdapter;
    private ArrayAdapter<Device> mSensorAdapter;
    private ArrayAdapter<Device> mActuatorAdapter;

    private List<Board> mBoardList;
    private List<String> mTriggerTypeList;
    private List<Device> mActuatorList;
    private List<Device> mSensorList;

    /**
     * Constructor
     */
    public AddScheduleFragment() {
        // Required empty public constructor
    }

    /**
     * Fragment new instance
     *
     * @return AddScheduleFragment instance
     */
    public static AddScheduleFragment newInstance() {
        return new AddScheduleFragment();
    }

    /**
     * onAttach method
     *
     * @param context application context
     */
    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        mContext = context;
    }

    /**
     * onCreate method
     *
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mApiService = new SmartGardenAdapter(mContext);
        mSessionManager = new SessionManager(mContext);

        mTriggerTypeList = new ArrayList<>();
        mBoardList = new ArrayList<>();
        mActuatorList = new ArrayList<>();
        mSensorList = new ArrayList<>();

        mTriggerTypeList.add(UiConstants.TRIGGER_DATE);
        mTriggerTypeList.add(UiConstants.TRIGGER_VALUE);
    }

    /**
     * onCreateView method
     *
     * @param inflater           layout inflater
     * @param container          viewgroup container
     * @param savedInstanceState saved instance state
     * @return inflated view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_add_schedule, container, false);

        mLoadingFrame = (FrameLayout) view.findViewById(R.id.loadingFrame);
        mInputIdentification = (TextInputLayout) view.findViewById(R.id.layoutIdentification);
        mInputName = (TextInputLayout) view.findViewById(R.id.layoutName);
        mBoardSpinner = (Spinner) view.findViewById(R.id.spinnerBoard);
        mActuatorSpinner = (Spinner) view.findViewById(R.id.spinnerActuator);
        mTriggerTypeSpinner = (Spinner) view.findViewById(R.id.spinnerTriggerType);
        mSensorSpinner = (Spinner) view.findViewById(R.id.spinnerSensor);
        mSensorLayout = (LinearLayout) view.findViewById(R.id.layoutSpinnerSensor);
        mRadioGroupOperator = (RadioGroup) view.findViewById(R.id.radioGroupOperator);
        mTriggerValue = (TextInputLayout) view.findViewById(R.id.layoutTriggerValue);
        mStartDate = (TextInputLayout) view.findViewById(R.id.layoutTriggerStartDate);
        mEndDate = (TextInputLayout) view.findViewById(R.id.layoutTriggerEndDate);

        Button autoIdentificator = view.findViewById(R.id.btnAutoId);
        Button saveButton = view.findViewById(R.id.btnSave);
        Button resetButton = view.findViewById(R.id.btnReset);

        ImageButton boardInfoButton = view.findViewById(R.id.btnInfoBoard);
        ImageButton actuatorInfoButton = view.findViewById(R.id.btnInfoActuator);
        ImageButton triggerTypeInfoButton = view.findViewById(R.id.btnInfoTriggerType);
        ImageButton sensorInfoButton = view.findViewById(R.id.btnInfoSensor);

        autoIdentificator.setOnClickListener(this);
        saveButton.setOnClickListener(this);
        resetButton.setOnClickListener(this);
        boardInfoButton.setOnClickListener(this);
        actuatorInfoButton.setOnClickListener(this);
        triggerTypeInfoButton.setOnClickListener(this);
        sensorInfoButton.setOnClickListener(this);

        mStartDate.getEditText().setOnClickListener(this);
        mEndDate.getEditText().setOnClickListener(this);

        mTriggerTypeSpinner.setOnItemSelectedListener(this);
        mBoardSpinner.setOnItemSelectedListener(this);

        return view;
    }

    /**
     * onViewCreated method
     *
     * @param view               created view
     * @param savedInstanceState saved instance state
     */
    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        mBoardAdapter = new ArrayAdapter<>(mContext,
                android.R.layout.simple_list_item_1, mBoardList);
        mBoardAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mBoardSpinner.setAdapter(mBoardAdapter);

        ArrayAdapter<String> mTriggerAdapter = new ArrayAdapter<>(mContext,
                android.R.layout.simple_list_item_1, mTriggerTypeList);
        mTriggerAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mTriggerTypeSpinner.setAdapter(mTriggerAdapter);

        mSensorAdapter = new ArrayAdapter<>(mContext,
                android.R.layout.simple_list_item_1, mSensorList);
        mSensorAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mSensorSpinner.setAdapter(mSensorAdapter);

        mActuatorAdapter = new ArrayAdapter<>(mContext,
                android.R.layout.simple_list_item_1, mActuatorList);
        mActuatorAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mActuatorSpinner.setAdapter(mActuatorAdapter);

        showLoadingFrame();
        getApiBoards();
    }

    /**
     * onClick method
     *
     * @param v clicked view
     */
    @Override
    public void onClick(View v) {

        switch (v.getId()) {
            case R.id.btnAutoId:
                onAutoIdButtonClicked();
                break;
            case R.id.btnSave:
                onSaveButtonClicked();
                break;
            case R.id.btnReset:
                onResetButtonClicked();
                break;
            case R.id.btnInfoBoard:
                onBoardInfoButtonClicked();
                break;
            case R.id.btnInfoActuator:
                onActuatorInfoButtonClicked();
                break;
            case R.id.btnInfoTriggerType:
                onTriggerTypeInfoButtonClicked();
                break;
            case R.id.btnInfoSensor:
                onSensorInfoButtonClicked();
                break;
            case R.id.inputStartDate:
                onStartDateClicked();
                break;
            case R.id.inputEndDate:
                onEndDateClicked();
                break;
        }
    }

    /**
     * onItemSelected method
     *
     * @param parent   parent view
     * @param view     selected view
     * @param position item position
     * @param id       item id
     */
    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
        switch (parent.getId()) {
            case R.id.spinnerTriggerType:
                onTriggerTypeSpinnerSelectionChange(position);
                break;
            case R.id.spinnerBoard:
                onBoardSpinnerSelectionChange(position);
                break;
        }
    }

    @Override
    public void onNothingSelected(AdapterView<?> parent) {

    }

    /**
     * Get api boards request
     */
    private void getApiBoards() {
        Call<List<Board>> boardsCall = mApiService.getBoards();
        boardsCall.enqueue(new Callback<List<Board>>() {
            @Override
            public void onResponse(Call<List<Board>> call, Response<List<Board>> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_OK:
                        if (response.body() != null) {
                            mBoardList = response.body();

                            mBoardAdapter.clear();
                            mBoardAdapter.addAll(mBoardList);
                            mBoardAdapter.notifyDataSetChanged();
                        }
                        showToast(getString(R.string.message_data_loaded));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<List<Board>> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.message_error_getting_boards));
            }
        });
    }

    /**
     * Post api schedule request
     *
     * @param schedule schedule to post
     */
    private void postApiSchedule(Schedule schedule) {
        Call<PostResponse> addScheduleCall = mApiService.addSchedule(schedule);
        addScheduleCall.enqueue(new Callback<PostResponse>() {
            @Override
            public void onResponse(Call<PostResponse> call, Response<PostResponse> response) {
                hideLoadingFrame();

                switch (response.code()) {
                    case ApiConstants.API_CODE_CREATED:
                        showToast(getString(R.string.message_schedule_added));
                        clearFields();
                        break;
                    case ApiConstants.API_CODE_CONFLICT:
                        showToast(getString(R.string.message_existing_schedule));
                        break;
                    case ApiConstants.API_CODE_UNAUTHORIZED:
                        showToast(getString(R.string.message_error_authorizing));
                        break;
                    case ApiConstants.API_CODE_INTERNAL_SERVER_ERROR:
                        showToast(getString(R.string.message_error_server));
                        break;
                }
            }

            @Override
            public void onFailure(Call<PostResponse> call, Throwable t) {
                hideLoadingFrame();
                showToast(getString(R.string.error_adding_schedule));
            }
        });
    }

    /**
     * Callback for setting automatic id
     */
    private void onAutoIdButtonClicked() {
        mInputIdentification.getEditText().setText(Utils.getAutomaticId());
    }

    /**
     * Callback for save button clicked
     */
    private void onSaveButtonClicked() {
        String id = mInputIdentification.getEditText().getText().toString();
        String name = mInputName.getEditText().getText().toString();
        Board board = (Board) mBoardSpinner.getSelectedItem();
        Device actuator = (Device) mActuatorSpinner.getSelectedItem();
        String triggerType = (String) mTriggerTypeSpinner.getSelectedItem();

        if (TextUtils.isEmpty(id)) {
            mInputIdentification.setError(getString(R.string.message_required_field));
            return;
        } else {
            mInputIdentification.setError(null);
        }

        if (TextUtils.isEmpty(name)) {
            mInputName.setError(getString(R.string.message_required_field));
            return;
        } else {
            mInputName.setError(null);
        }

        if (board == null) {
            showToast(getString(R.string.message_board_must_be_selected));
            return;
        }

        if (actuator == null) {
            showToast(getString(R.string.message_actuator_must_be_selected));
            return;
        }

        if (triggerType == null) {
            showToast(getString(R.string.message_trigger_type_must_be_selected));
            return;
        }

        Parameter parameter = new Parameter();

        if (triggerType.equals(UiConstants.TRIGGER_DATE)) {
            String startDate = (String) mStartDate.getEditText().getText().toString();
            String endDate = (String) mEndDate.getEditText().getText().toString();

            if (TextUtils.isEmpty(startDate)) {
                mStartDate.setError(getString(R.string.message_required_field));
                return;
            } else {
                mStartDate.setError(null);
            }

            if (TextUtils.isEmpty(endDate)) {
                mEndDate.setError(getString(R.string.message_required_field));
                return;
            } else {
                mEndDate.setError(null);
            }

            if (!Utils.checkTimes(startDate, endDate)) {
                mEndDate.setError(getString(R.string.message_end_date_before_start_date));
                return;
            }

            parameter.setStartTime(startDate);
            parameter.setEndTime(endDate);
        } else {
            Device sensor = (Device) mSensorSpinner.getSelectedItem();

            int checkedOperator = mRadioGroupOperator.getCheckedRadioButtonId();
            String value = (String) mTriggerValue.getEditText().getText().toString();
            String operator = "";

            switch (checkedOperator) {
                case R.id.rbBigger:
                    operator = ">";
                    break;
                case R.id.rbLower:
                    operator = "<";
                    break;
                default:
                    showToast(getString(R.string.message_operator_must_be_selected));
                    return;
            }

            if (sensor == null) {
                showToast(getString(R.string.message_sensor_must_be_selected));
                return;
            }

            if (TextUtils.isEmpty(value)) {
                mTriggerValue.setError(getString(R.string.message_required_field));
                return;
            } else {
                if (!Utils.checkTriggerValue(Integer.parseInt(value))) {
                    mTriggerValue.setError(getString(R.string.message_value_greater_than));
                    return;
                } else {
                    mTriggerValue.setError(null);
                }
            }

            parameter.setSensorId(sensor.getId());
            parameter.setSensorName(sensor.getName());
            parameter.setOperator(operator);
            parameter.setValue(Integer.parseInt(value));
        }

        Schedule schedule = new Schedule();
        schedule.setId(id);
        schedule.setUserId(mSessionManager.getUserId());
        schedule.setActuatorId(actuator.getId());
        schedule.setActuatorName(actuator.getName());
        schedule.setBoardId(board.getId());
        schedule.setName(name);
        schedule.setType(triggerType);
        schedule.setParameters(parameter);

        showLoadingFrame();
        postApiSchedule(schedule);
    }

    /**
     * Callback for reset button clicked
     */
    private void onResetButtonClicked() {
        clearFields();
    }

    /**
     * Callback for board info button clicked
     */
    private void onBoardInfoButtonClicked() {
        showToast(getString(R.string.message_info_board));
    }

    /**
     * Callback for actuator info button clicked
     */
    private void onActuatorInfoButtonClicked() {
        showToast(getString(R.string.message_info_actuator));
    }

    /**
     * Callback for trigger info button clicked
     */
    private void onTriggerTypeInfoButtonClicked() {
        showToast(getString(R.string.message_info_trigger_types));
    }

    /**
     * Callback for sensor info button clicked
     */
    private void onSensorInfoButtonClicked() {
        showToast(getString(R.string.message_info_sensor));
    }

    /**
     * Callback for start date field clicked
     */
    private void onStartDateClicked() {
        Calendar cldr = Calendar.getInstance();
        int hour = cldr.get(Calendar.HOUR_OF_DAY);
        int minutes = cldr.get(Calendar.MINUTE);

        mTimePicker = new TimePickerDialog(mContext, new TimePickerDialog.OnTimeSetListener() {
            @Override
            public void onTimeSet(TimePicker tp, int sHour, int sMinute) {
                mStartDate.getEditText().setText(String.format("%02d:%02d", sHour, sMinute));
            }
        }, hour, minutes, true);
        mTimePicker.show();
    }

    /**
     * Callback for end date field clicked
     */
    private void onEndDateClicked() {
        Calendar cldr = Calendar.getInstance();
        int hour = cldr.get(Calendar.HOUR_OF_DAY);
        int minutes = cldr.get(Calendar.MINUTE);

        mTimePicker = new TimePickerDialog(mContext, new TimePickerDialog.OnTimeSetListener() {
            @Override
            public void onTimeSet(TimePicker tp, int sHour, int sMinute) {
                mEndDate.getEditText().setText(String.format("%02d:%02d", sHour, sMinute));
            }
        }, hour, minutes, true);
        mTimePicker.show();
    }

    /**
     * Callback for trigger type spinner selection change
     *
     * @param position selected spinner trigger type position
     */
    private void onTriggerTypeSpinnerSelectionChange(int position) {
        clearDates();
        clearValues();

        if (mTriggerTypeList.get(position).equals(UiConstants.TRIGGER_DATE)) {
            mRadioGroupOperator.setVisibility(View.GONE);
            mTriggerValue.setVisibility(View.GONE);
            mSensorLayout.setVisibility(View.GONE);

            mStartDate.setVisibility(View.VISIBLE);
            mEndDate.setVisibility(View.VISIBLE);
        } else {
            mRadioGroupOperator.setVisibility(View.VISIBLE);
            mTriggerValue.setVisibility(View.VISIBLE);
            mSensorLayout.setVisibility(View.VISIBLE);

            mStartDate.setVisibility(View.GONE);
            mEndDate.setVisibility(View.GONE);
        }
    }

    /**
     * Callback for board spinner selection change
     *
     * @param position selected spinner board position
     */
    private void onBoardSpinnerSelectionChange(int position) {
        List<Device> deviceList = mBoardList.get(position).getDevices();

        mSensorList = new ArrayList<>();
        mActuatorList = new ArrayList<>();

        for (Device device : deviceList) {
            if (device.getType().equals(UiConstants.SENSOR_TAG)) {
                mSensorList.add(device);
            } else {
                mActuatorList.add(device);
            }
        }

        mSensorAdapter.clear();
        mSensorAdapter.addAll(mSensorList);
        mSensorAdapter.notifyDataSetChanged();

        mActuatorAdapter.clear();
        mActuatorAdapter.addAll(mActuatorList);
        mActuatorAdapter.notifyDataSetChanged();
    }

    /**
     * Show loading frame
     */
    private void showLoadingFrame() {
        mLoadingFrame.setVisibility(View.VISIBLE);
    }

    /**
     * Hide loading frame
     */
    private void hideLoadingFrame() {
        mLoadingFrame.setVisibility(View.GONE);
    }

    /**
     * Show toast with message
     *
     * @param message data to show
     */
    private void showToast(String message) {
        Toast.makeText(mContext, message, Toast.LENGTH_SHORT).show();
    }

    /**
     * Clear dates fields
     */
    private void clearDates() {
        mStartDate.getEditText().getText().clear();
        mEndDate.getEditText().getText().clear();
    }

    /**
     * Clear values fields
     */
    private void clearValues() {
        mRadioGroupOperator.clearCheck();
        mTriggerValue.getEditText().getText().clear();
        if (mSensorSpinner.getCount() > 0) {
            mSensorSpinner.setSelection(0);
        }
    }

    /**
     * Clear all editable fields
     */
    private void clearFields() {
        mInputIdentification.getEditText().getText().clear();
        mInputName.getEditText().getText().clear();

        if (mBoardSpinner.getCount() > 0) {
            mBoardSpinner.setSelection(0);
        }

        if (mTriggerTypeSpinner.getCount() > 0) {
            mTriggerTypeSpinner.setSelection(0);
        }

        clearDates();
        clearValues();
    }
}
