const express = require('express');
const morgan = require('morgan')
const bodyParser = require('body-parser');
const jwt = require('jsonwebtoken');

const config = require('./config');
const middleware = require('./middleware');
const dbHelper = require('./dbHelper');

const app = express();

app.use(morgan('dev'))

app.use(bodyParser.urlencoded({
    extended: false
}));
app.use(bodyParser.json());

dbHelper.initConnection().then(dbClient => {
    const db = dbClient.db("test");

    app.listen(3000, () => {
        console.log("API server running...");
    })

    /***************************************************************************
     * LOGIN REQUEST
     ***************************************************************************/
    app.post('/login', function (req, res) {
        dbHelper.getUser(db, req.body.username)
            .then(user => {
                if (user != null) {
                    if (user.password === req.body.password) {
                        var token = jwt.sign({
                                id: user._id,
                            },
                            config.secret, {
                                expiresIn: '365d'
                            }
                        );
                        res.status(200).json({
                            success: true,
                            message: "Login successfully",
                            id: user._id,
                            token: token
                        })
                    } else {
                        res.status(401).json({
                            success: false,
                            message: "Invalid password"
                        })
                    }
                } else {
                    res.status(401).json({
                        success: false,
                        message: "Invalid user"
                    })
                }
            })
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            })
    });

    /***************************************************************************
     * REGISTER REQUEST
     ***************************************************************************/
    app.post('/register', function (req, res) {
        dbHelper.getUser(db, req.body.username)
            .then(user => {
                if (user) {
                    res.status(409).json({
                        success: false,
                        message: "User already exist"
                    })
                } else {
                    dbHelper.setUser(db, req.body)
                        .then(
                            res.status(201).json({
                                success: true,
                                message: "User created successfully"
                            })
                        )
                }
            })
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            });
    });

    /***************************************************************************
     * ADD DRIVER REQUEST
     ***************************************************************************/
    app.post('/addDriver', function (req, res) {
        dbHelper.setDriver(db, req.body)
            .then(
                res.status(201).json({
                    success: true,
                    message: "Driver added successfully"
                })
            )
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            });
    });

    /***************************************************************************
     * DRIVERS REQUEST
     ***************************************************************************/
    app.get('/drivers', middleware.checkToken, function (req, res) {
        dbHelper.getDrivers(db)
            .then(drivers => {
                res.status(200).json(drivers);
            })
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            })
    });

    /***************************************************************************
     * DRIVER REQUEST
     ***************************************************************************/
    app.get('/driver', middleware.checkToken, function (req, res) {
        dbHelper.getDriver(db, req.query.id)
            .then(driver => {
                res.status(200).json(driver);
            })
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            })
    });

    /***************************************************************************
     * ADD BOARD REQUEST
     ***************************************************************************/
    app.post('/addBoard', middleware.checkToken, function (req, res) {
        dbHelper.checkBoardId(db, req.body._id)
            .then(count => {
                if (count > 0) {
                    res.status(409).json({
                        success: false,
                        message: "Duplicated id"
                    })
                } else {
                    dbHelper.setBoard(db, req.body)
                        .then(
                            res.status(201).json({
                                success: true,
                                message: "Board added successfully"
                            })
                        )
                }
            })
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            });
    });

    /***************************************************************************
     * DELETE BOARDS REQUEST
     ***************************************************************************/
    app.delete('/deleteBoard', middleware.checkToken, function (req, res) {
        dbHelper.deleteBoard(db, req.query.id)
            .then(
                dbHelper.deleteSchedulesByBoard(db, req.query.id)
                    .then(
                        res.status(200).json({
                            success: true,
                            message: "Board deleted successfully"
                        })
                    )
            )
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            })
    });

    /***************************************************************************
     * BOARDS REQUEST
     ***************************************************************************/
    app.get('/boards', middleware.checkToken, function (req, res) {
        dbHelper.getBoards(db, req.decoded.id)
            .then(boards => {
                res.status(200).json(boards);
            })
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            })
    });

    /***************************************************************************
     * ADD DEVICE REQUEST
     ***************************************************************************/
    app.post('/addDevice', middleware.checkToken, function (req, res) {
        dbHelper.checkDeviceId(db, req.body._id)
            .then(count => {
                if (count > 0) {
                    res.status(409).json({
                        success: false,
                        message: "Duplicated id"
                    })
                } else {
                    dbHelper.setDevice(db, req.body)
                        .then(
                            res.status(201).json({
                                success: true,
                                message: "Device added successfully"
                            })
                        )
                }
            })
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            });
    });

    /***************************************************************************
     * DELETE DEVICE REQUEST
     ***************************************************************************/
    app.delete('/deleteDevice', middleware.checkToken, function (req, res) {
        dbHelper.deleteDevice(db, req.query.boardId, req.query.deviceId)
            .then(
                dbHelper.deleteSchedulesByDevice(db, req.query.deviceId)
                    .then(
                        res.status(200).json({
                            success: true,
                            message: "Device deleted successfully"
                        })
                    )
            )
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            })
    });

    /***************************************************************************
     * ADD DEVICE VALUE REQUEST
     ***************************************************************************/
    app.post('/addDeviceValue', middleware.checkToken, function (req, res) {
        dbHelper.setDeviceValue(db, req.body.device_id, req.body.value)
            .then(
                res.status(201).json({
                    success: true,
                    message: "Device value added successfully"
                })
            )
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                })
            })
    });

    /***************************************************************************
     * ADD SCHEDULE REQUEST
     ***************************************************************************/
    app.post('/addSchedule', middleware.checkToken, function (req, res) {
        dbHelper.checkScheduleId(db, req.body._id)
            .then(count => {
                if (count > 0) {
                    res.status(409).json({
                        success: false,
                        message: "Duplicated id"
                    })
                } else {
                    dbHelper.setSchedule(db, req.body)
                        .then(
                            res.status(201).json({
                                success: true,
                                message: "Schedule added successfully"
                            })
                        )
                }
            })
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            });
    });

    /***************************************************************************
     * DELETE SCHEDULE REQUEST
     ***************************************************************************/
    app.delete('/deleteSchedule', middleware.checkToken, function (req, res) {
        dbHelper.deleteScheduleById(db, req.query.id)
            .then(
                res.status(200).json({
                    success: true,
                    message: "Schedule deleted successfully"
                })
            )
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            })
    });

    /***************************************************************************
     * SCHEDULES REQUEST
     ***************************************************************************/
    app.get('/schedules', middleware.checkToken, function (req, res) {
        dbHelper.getSchedules(db, req.decoded.id)
            .then(schedules => {
                res.status(200).json(schedules);
            })
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            })
    })

    /***************************************************************************
     * ADD LOG REQUEST
     ***************************************************************************/
    app.post('/addLog', middleware.checkToken, function (req, res) {
        dbHelper.setLog(db, req.body)
            .then(
                res.status(201).json({
                    success: true,
                    message: "Log added successfully"
                })
            )
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            });
    })

    /***************************************************************************
     * LOGS REQUEST
     ***************************************************************************/
    app.get('/logs', middleware.checkToken, function (req, res) {
        dbHelper.getLogs(db, req.decoded.id)
            .then(logs => {
                res.status(200).json(logs);
            })
            .catch(e => {
                res.status(500).json({
                    success: false,
                    message: "Internal server error"
                });
            })
    })

    /***************************************************************************
     * CLOSE DATABASE CONNECTION WHEN QUIT SERVER
     ***************************************************************************/
    process.on('SIGINT', () => {
        dbHelper.finishConnection(dbClient).then(()=> {
          console.log("Database connection closed");
          process.exit();
        })
    });
    
    process.on('SIGTERM', () => {
        dbHelper.finishConnection(dbClient).then(()=> {
          console.log("Database connection closed");
          process.exit();
        })
    });
})


